unit Attracts;

interface

uses Classes;
 

type
  TAttrVisitor = class;
  
  TAttraction = class (TObject)
  private
    FName: PString;
  protected
    function GetDuration: Integer; virtual; abstract;
    function GetName: String;
    function GetPrice: Integer; virtual; abstract;
    procedure SetName(const Value: String);
  public
    destructor Destroy; override;
    procedure AcceptAttrVisitor(Visitor: TAttrVisitor); virtual;
    property Duration: Integer read GetDuration;
    property Name: String read GetName write SetName;
    property Price: Integer read GetPrice;
  end;

  TSingleAttraction = class (TAttraction)
  private
    FDuration: Integer;
    FPrice: Integer;
  protected
    function GetDuration: Integer; override;
    function GetPrice: Integer; override;
  public
    procedure AcceptAttrVisitor(Visitor: TAttrVisitor); override;
  end;

  TCycleTour = class (TSingleAttraction)
  public
    constructor Create;
    procedure AcceptAttrVisitor(Visitor: TAttrVisitor); override;
  end;

  TRedLightsTour = class (TSingleAttraction)
  public
    constructor Create;
    procedure AcceptAttrVisitor(Visitor: TAttrVisitor); override;
  end;

  TTulipsTour = class (TSingleAttraction)
  public
    constructor Create;
    procedure AcceptAttrVisitor(Visitor: TAttrVisitor); override;
  end;

  TWindmillTour = class (TSingleAttraction)
  public
    constructor Create;
    procedure AcceptAttrVisitor(Visitor: TAttrVisitor); override;
  end;

  TExcursion = class (TAttraction)
  private
    FAttractions: TList;
  protected
    function GetAttractions(Index: Integer): TAttraction;
    function GetAttrCount: Integer;
    function GetDuration: Integer; override;
    function GetPrice: Integer; override;
  public
    constructor Create;
    destructor Destroy; override;
    procedure AcceptAttrVisitor(Visitor: TAttrVisitor); override;
    property Attractions[Index: Integer] : TAttraction read GetAttractions; default;
    property AttrCount: Integer read GetAttrCount;
  end;

  TDutchActive = class (TExcursion)
  public
    constructor Create;
    procedure AcceptAttrVisitor(Visitor: TAttrVisitor); override;
  end;

  TDutchClassics = class (TExcursion)
  public
    constructor Create;
    procedure AcceptAttrVisitor(Visitor: TAttrVisitor); override;
  end;

  TAttrVisitor = class (TObject)
  public
    procedure VisitAttraction(Instance: TAttraction); virtual;
    procedure VisitCycleTour(Instance: TCycleTour); virtual;
    procedure VisitDutchActive(Instance: TDutchActive); virtual;
    procedure VisitDutchClassics(Instance: TDutchClassics); virtual;
    procedure VisitExcursion(Instance: TExcursion); virtual;
    procedure VisitRedLightsTour(Instance: TRedLightsTour); virtual;
    procedure VisitSingleAttraction(Instance: TSingleAttraction); virtual;
    procedure VisitTulipsTour(Instance: TTulipsTour); virtual;
    procedure VisitWindmillTour(Instance: TWindmillTour); virtual;
  end;


implementation

uses MMUtils, SysUtils;

{
************************************** TAttraction **************************************
}
destructor TAttraction.Destroy;
begin
  DisposeStr(FName);
  inherited Destroy;
end;

procedure TAttraction.AcceptAttrVisitor(Visitor: TAttrVisitor);
begin
  Visitor.VisitAttraction(Self);
end;

function TAttraction.GetName: String;
begin
  Result := StringValue(FName);
end;

procedure TAttraction.SetName(const Value: String);
begin
  AssignStr(FName, Value);
end;

{
*********************************** TSingleAttraction ***********************************
}
procedure TSingleAttraction.AcceptAttrVisitor(Visitor: TAttrVisitor);
begin
  Visitor.VisitSingleAttraction(Self);
end;

function TSingleAttraction.GetDuration: Integer;
begin
  Result := FDuration;
end;

function TSingleAttraction.GetPrice: Integer;
begin
  Result := FPrice;
end;

{
*************************************** TCycleTour ***************************************
}
constructor TCycleTour.Create;
begin
  inherited Create;
  Name := 'Cycling tour through the national park "De hoge veluwe"';
  FPrice := 250;
  FDuration := 48;
end;

procedure TCycleTour.AcceptAttrVisitor(Visitor: TAttrVisitor);
begin
  Visitor.VisitCycleTour(Self);
end;

{
************************************* TRedLightsTour *************************************
}
constructor TRedLightsTour.Create;
begin
  inherited Create;
  Name := 'Exclusive tour through the Amsterdam Red light district';
  FPrice := 2500;
  FDuration := 6;
end;

procedure TRedLightsTour.AcceptAttrVisitor(Visitor: TAttrVisitor);
begin
  Visitor.VisitRedLightsTour(Self);
end;

{
************************************** TTulipsTour **************************************
}
constructor TTulipsTour.Create;
begin
  inherited Create;
  Name := 'Bus tour to the Keukenhof and tulip fields ';
  FPrice := 48;
  FDuration := 12;
end;

procedure TTulipsTour.AcceptAttrVisitor(Visitor: TAttrVisitor);
begin
  Visitor.VisitTulipsTour(Self);
end;

{
************************************* TWindmillTour *************************************
}
constructor TWindmillTour.Create;
begin
  inherited Create;
  Name := 'Bus tour along windmills and a visit to craftsman shops';
  FPrice := 100;
  FDuration := 4;
end;

procedure TWindmillTour.AcceptAttrVisitor(Visitor: TAttrVisitor);
begin
  Visitor.VisitWindmillTour(Self);
end;

{
*************************************** TExcursion ***************************************
}
constructor TExcursion.Create;
begin
  inherited Create;
  FAttractions := TList.Create;
end;

destructor TExcursion.Destroy;
var
  I: Integer;
begin
  for I := 0 to AttrCount - 1 do
    Attractions[I].Free;
  FAttractions.Free;
  inherited Destroy;
end;

procedure TExcursion.AcceptAttrVisitor(Visitor: TAttrVisitor);
begin
  Visitor.VisitExcursion(Self);
end;

function TExcursion.GetAttractions(Index: Integer): TAttraction;
begin
  Result := TAttraction(FAttractions[Index]);
end;

function TExcursion.GetAttrCount: Integer;
begin
  Result := FAttractions.Count;
end;

function TExcursion.GetDuration: Integer;
var
  I: Integer;
begin
  Result := 0;
  for I := 0 to AttrCount - 1 do
    Result := Result + Attractions[I].Duration;
end;

function TExcursion.GetPrice: Integer;
var
  I: Integer;
begin
  Result := 0;
  for I := 0 to AttrCount - 1 do
    Result := Result + Attractions[I].Price;
end;

{
************************************** TDutchActive **************************************
}
constructor TDutchActive.Create;
begin
  inherited Create;
  FAttractions.Add(TCycleTour.Create);
  FAttractions.Add(TRedLightsTour.Create);
  FAttractions.Add(TWindmillTour.Create);
  Name := 'Dutch classics tour';
end;

procedure TDutchActive.AcceptAttrVisitor(Visitor: TAttrVisitor);
begin
  Visitor.VisitDutchActive(Self);
end;

{
************************************* TDutchClassics *************************************
}
constructor TDutchClassics.Create;
begin
  inherited Create;
  FAttractions.Add(TTulipsTour.Create);
  FAttractions.Add(TWindmillTour.Create);
  FAttractions.Add(TRedLightsTour.Create);
  Name := 'Dutch active tour';
end;

procedure TDutchClassics.AcceptAttrVisitor(Visitor: TAttrVisitor);
begin
  Visitor.VisitDutchClassics(Self);
end;

{
************************************** TAttrVisitor **************************************
}
procedure TAttrVisitor.VisitAttraction(Instance: TAttraction);
begin
end;

procedure TAttrVisitor.VisitCycleTour(Instance: TCycleTour);
begin
end;

procedure TAttrVisitor.VisitDutchActive(Instance: TDutchActive);
begin
end;

procedure TAttrVisitor.VisitDutchClassics(Instance: TDutchClassics);
begin
end;

procedure TAttrVisitor.VisitExcursion(Instance: TExcursion);
var
  I: Integer;
begin
  for I := 0 to Instance.AttrCount - 1 do
    Instance[I].AcceptAttrVisitor(Self);
end;

procedure TAttrVisitor.VisitRedLightsTour(Instance: TRedLightsTour);
begin
end;

procedure TAttrVisitor.VisitSingleAttraction(Instance: TSingleAttraction);
begin
end;

procedure TAttrVisitor.VisitTulipsTour(Instance: TTulipsTour);
begin
end;

procedure TAttrVisitor.VisitWindmillTour(Instance: TWindmillTour);
begin
end;


initialization
end.
