unit Filters;

interface

uses Classes, SysUtils;

type
  TStreamable = class;

  TFilter = class(TStream)
  private
    FOwnsStream: Boolean;
    FStream: TStream;
  protected
    procedure RegisterInstance(Instance: TStreamable); virtual;
    procedure SetStream(Value: TStream); virtual;
  public
    constructor Create(AStream: TStream);
    destructor Destroy; override;
    function Get: TStreamable; virtual;
    procedure Put(Instance: TStreamable); virtual;
    function Read(var Buffer; Count: Longint): Longint; override;
    function ReadStr: String;
    function Seek(Offset: Longint; Origin: Word): Longint; override;
    procedure Truncate; virtual;
    function Write(const Buffer; Count: Longint): Longint; override;
    procedure WriteStr(const Value: string);
    property OwnsStream: Boolean read FOwnsStream write FOwnsStream;
    property Stream: TStream read FStream write SetStream;
 end;


  TStreamable = class(TPersistent)
    constructor Load(S: TFilter); virtual;
    procedure Store(S: TFilter); virtual;
  end;

  TStreamableClass = class of TStreamable;


type
  TCycleFilter = class(TFilter)
  private
    FReferences: TList;
  protected
    procedure RegisterInstance(Instance: TStreamable); override;
  public
    constructor Create(AStream: TStream);
    destructor Destroy; override;
    function Get: TStreamable; override;
    procedure Put(Instance: TStreamable); override;
  end;

{ Streamable registration exception class }

  EStreamableException = class(Exception);

  function CopyStreamable(Instance: TStreamable): TStreamable;

  procedure LoadStrings(Filter: TFilter; List: TStrings);

  procedure StoreStrings(Filter: TFilter; List: TStrings);

  function LoadPChar(Stream: TStream): PChar;

  procedure StorePChar(Stream: TStream; Source: PChar);

  function FindStreamable(const ClassName: String): TStreamableClass;

  function GetStreamable(const ClassName: String): TStreamableClass;

  procedure RegisterStreamable(AClass: TStreamableClass);

  procedure RegisterStreamables(AClasses: array of TStreamableClass);

  procedure UnregisterStreamables; far;

implementation

uses WinTypes, WinProcs, MMUtils;
{
*********************** Streamable registration ******************************
}
var
  StreamableList: TList;

procedure StreamableNotFound(const ClassName: string);
begin
  raise EStreamableException.Create(ClassName + ' not registered as streamable');
end;

function FindStreamable(const ClassName: string): TStreamableClass;
begin
  Result := GetStreamable(ClassName);
  if Result = nil then StreamableNotFound(ClassName);
end;

function GetStreamable(const ClassName: string): TStreamableClass;
var
  I: Integer;
begin
  for I := 0 to StreamableList.Count - 1 do
  begin
    Result := StreamableList[I];
    if CompareText(Result.ClassName, ClassName) = 0 then Exit;
  end;
  Result := nil;
end;

procedure RegisterStreamable(AClass: TStreamableClass);
var
  ClassName: string[63];
begin
  if not Assigned(StreamableList) then
    StreamableList := TList.Create;
  while StreamableList.IndexOf(AClass) = -1 do
  begin
    ClassName := AClass.ClassName;
    if GetStreamable(ClassName) = nil then
      StreamableList.Add(AClass);
    if AClass = TStreamable then Break;
    AClass := TStreamableClass(AClass.ClassParent);
  end;
end;

procedure RegisterStreamables(AClasses: array of TStreamableClass);
var
  I: Integer;
begin
  for I := Low(AClasses) to High(AClasses) do RegisterStreamable(AClasses[I]);
end;

procedure UnregisterStreamables;
begin
  if Assigned(StreamableList) then
  begin
    StreamableList.Clear;
    StreamableList := nil;
  end;
end;

function CopyStreamable(Instance: TStreamable): TStreamable;
var S: TStream;
    F: TFilter;
begin
  Result := nil;
  S := TMemoryStream.Create;
  try
    F := TFilter.Create(S);
    try
      F.Put(Instance);
      F.Seek(0, 0);
      Result := F.Get;
    finally
      F.Free;
    end;
  finally
    S.Free;
  end;
end;

procedure LoadStrings(Filter: TFilter; List: TStrings);
var I, LineCnt: Integer;
begin
  List.Clear;
  Filter.Read(LineCnt, SizeOf(LineCnt));
  for I := 0 to LineCnt - 1 do
    List.Add(Filter.ReadStr);
end;

procedure StoreStrings(Filter: TFilter; List: TStrings);
var I, LineCnt: Integer;
begin
  LineCnt := List.Count;
  Filter.Write(LineCnt, SizeOf(LineCnt));
  for I := 0 to LineCnt - 1 do
    Filter.WriteStr(List[I]);
end;

function LoadPChar(Stream: TStream): PChar;
var Cnt: Word;
begin
  Stream.Read(Cnt, SizeOf(Cnt));
  if Cnt > 0 then
  begin
    Inc(Cnt);
    Result := StrAlloc(Cnt);
    Stream.Read(Result[0], Cnt);
  end
  else
    Result := nil;
end;

procedure StorePChar(Stream: TStream; Source: PChar);
var Cnt: Word;
begin
  Cnt := StrLen(Source);
  Stream.Write(Cnt, SizeOf(Cnt));
  if Cnt > 0 then
    Stream.Write(Source[0], Cnt + 1);
end;

{
****************************** TFilter ***************************************
}
constructor TFilter.Create(AStream: TStream);
begin
  inherited Create;
  Stream := AStream;
  FOwnsStream := False;
end;

destructor TFilter.Destroy;
begin
  Stream := nil; { calls SetStream }
  inherited Destroy;
end;

function TFilter.Get: TStreamable;
begin
  Result := FindStreamable(ReadStr).Load(Self);
end;

procedure TFilter.Put(Instance: TStreamable);
begin
  WriteStr(Instance.ClassName);
  Instance.Store(Self);
end;

function TFilter.Read(var Buffer; Count: Longint): Longint;
begin
  if Assigned(FStream) then
    Result := FStream.Read(Buffer, Count)
  else
    Read := 0;
end;

function TFilter.ReadStr: string;
begin
  Read(Result[0], 1);
  Read(Result[1], Ord(Result[0]));
end;

procedure TFilter.RegisterInstance(Instance: TStreamable);
begin
  if Assigned(FStream) and (FStream is TFilter) then
    TFilter(FStream).RegisterInstance(Instance);
end;

function TFilter.Seek(Offset: Longint; Origin: Word): Longint;
begin
  if Assigned(FStream) then
    Result := Stream.Seek(Offset, Origin)
  else
    Result := 0;
end;

procedure TFilter.SetStream(Value: TStream);
begin
  if FOwnsStream then FStream.Free; { Allright even if FStream = nil }
  FStream := Value;
end;

procedure TFilter.Truncate;
begin
end;

function TFilter.Write(const Buffer; Count: Longint): Longint;
begin
  if Assigned(FStream) then
    Result := Stream.Write(Buffer, Count)
  else
    Result := 0;
end;

procedure TFilter.WriteStr(const Value: string);
begin
  Write(Value[0], Ord(Value[0]) + 1);
end;

{
******************************** TStreamable *********************************
}
constructor TStreamable.Load(S: TFilter);
begin
  Create;
  S.RegisterInstance(Self);
end;

procedure TStreamable.Store(S: TFilter);
begin
end;

{
********************************* TCycleFilter *********************************
}
constructor TCycleFilter.Create(AStream: TStream);
begin
  inherited Create(AStream);
  FReferences := TList.Create;
end;

destructor TCycleFilter.Destroy;
begin
  FReferences.Free;
  inherited Destroy;
end;

function TCycleFilter.Get: TStreamable;
var
  Idx: Integer;
begin
  Read(Idx, SizeOf(Idx));
  if Idx = -1 then
  begin
    Idx := FReferences.Count;
    FReferences.Add(nil);
    FindStreamable(ReadStr).Load(Self);
  end;
  if Idx = -2 then
    Result := nil
  else
    Result := FReferences.Items[Idx];
end;

procedure TCycleFilter.Put(Instance: TStreamable);
var
  Idx: Integer;
begin
  if Assigned(Instance) then
    Idx := FReferences.IndexOf(Instance)
  else
    Idx := -2;
  Write(Idx, SizeOf(Idx));
  if Idx = -1 then
  begin
    FReferences.Add(Instance);
    inherited Put(Instance)
  end;
end;

procedure TCycleFilter.RegisterInstance(Instance: TStreamable);
begin
  FReferences.Items[FReferences.Count - 1] := Instance;
  inherited RegisterInstance(Instance);
end;


begin
  StreamableList := TList.Create;
  AddExitProc(UnregisterStreamables);
end.
