unit Toursim;

interface

uses Classes, ExtCtrls, Attracts, Tourists, Graphics;

type
  TExcursionKind = (exDutchActive, exDutchClassics);

  TTourSimulator = class (TComponent)
  private
    FAttrIdx: Integer;
    FExcursion: TExcursion;
    FExcursionKind: TExcursionKind;
    FOnCurAttrChange: TNotifyEvent;
    FOnExcursionChange: TNotifyEvent;
    FTimer: TTimer;
  protected
    function CalcSatisfaction(Tourist: TTourist): Integer;
    procedure CurAttrChange;
    procedure ExcursionChange;
    function GetAttrCount: Integer;
    function GetCurAttr: TAttraction;
    function GetDuration: Integer;
    function GetEnabled: Boolean;
    function GetInterval: Integer;
    function GetPrice: Integer;
    procedure SetAttrIdx(Value: Integer);
    procedure SetEnabled(Value: Boolean);
    procedure SetExcursion(Value: TExcursion);
    procedure SetExcursionKind(Value: TExcursionKind);
    procedure SetInterval(Value: Integer);
    procedure TimerTimer(Sender: TObject);
    procedure UnwireTimer;
    procedure WireTimer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function BusinessSatisfaction: Integer;
    procedure First;
    function IsDone: Boolean;
    procedure Next;
    procedure Run;
    function StudentSatisfaction: Integer;
    property AttrCount: Integer read GetAttrCount;
    property AttrIdx: Integer read FAttrIdx write SetAttrIdx;
    property CurAttr: TAttraction read GetCurAttr;
    property Duration: Integer read GetDuration;
    property Excursion: TExcursion read FExcursion write SetExcursion;
    property Price: Integer read GetPrice;
  published
    property Enabled: Boolean read GetEnabled write SetEnabled;
    property ExcursionKind: TExcursionKind read FExcursionKind write SetExcursionKind;
    property Interval: Integer read GetInterval write SetInterval;
    property OnCurAttrChange: TNotifyEvent read FOnCurAttrChange write FOnCurAttrChange;
    property OnExcursionChange: TNotifyEvent read FOnExcursionChange write 
        FOnExcursionChange;
  end;

  TAttrDrawer = class (TAttrVisitor)
  private
    FImage: TImage;
  public
    procedure VisitCycleTour(Instance: TCycleTour); override;
    procedure VisitRedLightsTour(Instance: TRedLightsTour); override;
    procedure VisitTulipsTour(Instance: TTulipsTour); override;
    procedure VisitWindmillTour(Instance: TWindmillTour); override;
    property Image: TImage read FImage write FImage;
  end;


procedure Register;

implementation

uses MMUtils;

procedure Register;
begin
  RegisterComponents('MMDemo', [TTourSimulator]);
end;

{
************************************* TTourSimulator *************************************
}
constructor TTourSimulator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTimer := TTimer.Create(Self);
  Enabled := False;
  Interval := 3000;
  WireTimer;
  FExcursionKind := exDutchClassics;
  { SetExKind must detect change to assign new Excursion! }
  SetExcursionKind(exDutchActive);
  { This will actually instantiate the Excursion }
end;

destructor TTourSimulator.Destroy;
begin
  UnwireTimer;
  inherited Destroy;
end;

function TTourSimulator.BusinessSatisfaction: Integer;
var
  Businessman: TBusinessman;
begin
  Businessman := TBusinessman.Create;
  try
    Result := CalcSatisfaction(Businessman);
  finally
    Businessman.Free;
  end;
end;

function TTourSimulator.CalcSatisfaction(Tourist: TTourist): Integer;
var
  I: Integer;
begin
  if Assigned(FExcursion) then
  begin
    for I := 0 to FExcursion.AttrCount - 1 do
      FExcursion[I].AcceptAttrVisitor(Tourist);
    Result := Tourist.Satisfaction;
  end
  else
    Result := 0;
end;

procedure TTourSimulator.CurAttrChange;
begin
  if Assigned(FOnCurAttrChange) then FOnCurAttrChange(Self);
end;

procedure TTourSimulator.ExcursionChange;
begin
  if Assigned(FOnExcursionChange) then FOnExcursionChange(Self);
end;

procedure TTourSimulator.First;
begin
  SetAttrIdx(0);
end;

function TTourSimulator.GetAttrCount: Integer;
begin
  Result := FExcursion.AttrCount;
end;

function TTourSimulator.GetCurAttr: TAttraction;
begin
  if not IsDone then
    Result := FExcursion[FAttrIdx]
  else
    Result := nil;
end;

function TTourSimulator.GetDuration: Integer;
begin
  Result := FExcursion.Duration;
end;

function TTourSimulator.GetEnabled: Boolean;
begin
  Result := FTimer.Enabled;
end;

function TTourSimulator.GetInterval: Integer;
begin
  Result := FTimer.Interval;
end;

function TTourSimulator.GetPrice: Integer;
begin
  Result := FExcursion.Price;
end;

function TTourSimulator.IsDone: Boolean;
begin
  Result := (not Assigned(FExcursion)) or (FAttrIdx >= FExcursion.AttrCount);
end;

procedure TTourSimulator.Next;
begin
  SetAttrIdx(FAttrIdx + 1);
end;

procedure TTourSimulator.Run;
begin
  First;
  Enabled := True;
end;

procedure TTourSimulator.SetAttrIdx(Value: Integer);
begin
  if (Value <> FAttrIdx) and Assigned(FExcursion) and
     (Value < FExcursion.AttrCount) and (Value >= 0) then
  begin
    FAttrIdx := Value;
    CurAttrChange;
  end;
end;

procedure TTourSimulator.SetEnabled(Value: Boolean);
begin
  FTimer.Enabled := Value;
end;

procedure TTourSimulator.SetExcursion(Value: TExcursion);
begin
  if Value <> FExcursion then
  begin
    FExcursion.Free;
    FExcursion := Value;
  end;
end;

procedure TTourSimulator.SetExcursionKind(Value: TExcursionKind);
begin
  if Value <> FExcursionKind then
  begin
    Enabled := False;
    FExcursionKind := Value;
    case FExcursionKind of
      exDutchActive : SetExcursion(TDutchActive.Create);
      exDutchClassics : SetExcursion(TDutchClassics.Create);
    end;
    ExcursionChange;
    SetAttrIdx(0);
    CurAttrChange;
  end;
end;

procedure TTourSimulator.SetInterval(Value: Integer);
begin
  FTimer.Interval := Value;
end;

function TTourSimulator.StudentSatisfaction: Integer;
var
  Student: TStudent;
begin
  Student := TStudent.Create;
  try
    Result := CalcSatisfaction(Student);
  finally
    Student.Free;
  end;
end;

procedure TTourSimulator.TimerTimer(Sender: TObject);
begin
  Next;
  if IsDone then Enabled := False;
end;

procedure TTourSimulator.UnwireTimer;
begin
  FTimer.OnTimer := nil;
end;

procedure TTourSimulator.WireTimer;
begin
  FTimer.OnTimer := TimerTimer;
end;

{
************************************** TAttrDrawer **************************************
}
procedure TAttrDrawer.VisitCycleTour(Instance: TCycleTour);
begin
  { Draw an appropriate bitmap on the canvas here }
  Image.Picture.LoadFromFile(CommandPath + 'Cycle.BMP');
end;

procedure TAttrDrawer.VisitRedLightsTour(Instance: TRedLightsTour);
begin
  { Draw an appropriate bitmap on the canvas here }
  Image.Picture.LoadFromFile(CommandPath + 'RedLight.BMP');
end;

procedure TAttrDrawer.VisitTulipsTour(Instance: TTulipsTour);
begin
  { Draw an appropriate bitmap on the canvas here }
  Image.Picture.LoadFromFile(CommandPath + 'TULIPS.BMP');
end;

procedure TAttrDrawer.VisitWindmillTour(Instance: TWindmillTour);
begin
  { Draw an appropriate bitmap on the canvas here }
  Image.Picture.LoadFromFile(CommandPath + 'WINDMILL.BMP');
end;


initialization
end.
